/*
 * $Id: config.cpp,v 1.2 2004/05/01 22:50:34 fredrik Exp $
 * 
 * nVIDIA KWin client configuration module
 *
 * Copyright (C) 2002 Fredrik H�lund <fredrik@kde.org>
 *
 * Based on the Quartz configuration module,
 *     Copyright (c) 2001 Karol Szwed <gallium@kde.org>
 *  
 * nVIDIA KWin client (version 1.0a)
 *
 * Copyright (C) 2004 Jose Rafael Castillo <jrch99@cantv.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the license, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <kglobal.h>
#include <klocale.h>

#include <qcheckbox.h>
#include <qradiobutton.h>
#include <kcolorbutton.h>
#include <qcombobox.h>
#include <qslider.h>
#include <qcolor.h>
#include <qfiledialog.h>
#include <qlineedit.h>
#include <qdir.h>
#include <qlabel.h>
#include "config.h"
#include "config.moc"

extern "C"
  {
    QObject* allocate_config( KConfig* conf, QWidget* parent )
    {
      return ( new nVIDIAConfig( conf, parent ) );
    }
  }


/* NOTE:
 * 'conf' 	is a pointer to the kwindecoration modules open kwin config,
 *			and is by default set to the "Style" group.
 *
 * 'parent'	is the parent of the QObject, which is a VBox inside the
 *			Configure tab in kwindecoration
 */

nVIDIAConfig::nVIDIAConfig( KConfig* conf, QWidget* parent )
    : QObject( parent )
{
  KGlobal::locale()->insertCatalogue("kwin_nvidia_config");
  c = new KConfig( "kwinnvidiarc" );
  ui = new nVIDIAConfigUI( parent );
  connect( ui->comboBox1, SIGNAL(activated(const QString &)), this, SLOT(EnableURL(const QString &)));
  connect( ui->showAppIcons,    SIGNAL(clicked()), SIGNAL(changed()) );
  connect( ui->largeGrabBars,   SIGNAL(clicked()), SIGNAL(changed()) );
  connect( ui->useShadowedText, SIGNAL(clicked()), SIGNAL(changed()) );
  connect( ui->shadow1, SIGNAL(clicked()), SIGNAL(changed()) );
  connect( ui->shadow2, SIGNAL(clicked()), SIGNAL(changed()) );
  connect( ui->shadow3, SIGNAL(clicked()), SIGNAL(changed()) );
  connect( ui->left, SIGNAL(clicked()), SIGNAL(changed()) );
  connect( ui->right, SIGNAL(clicked()), SIGNAL(changed()) );
  connect( ui->center, SIGNAL(clicked()), SIGNAL(changed()) );
  connect( ui->systemcolor, SIGNAL(clicked()), SIGNAL(changed()) );
  connect( ui->originalcolor, SIGNAL(clicked()), SIGNAL(changed()) );
  connect( ui->effectvalue, SIGNAL(sliderReleased()), SIGNAL(changed()) );
  connect( ui->effectcolor, SIGNAL(clicked()), SIGNAL(changed()) );
  connect( ui->ShadowColor, SIGNAL(clicked()), SIGNAL(changed()) );
  connect( ui->togray, SIGNAL(clicked()), SIGNAL(changed()) );
  connect( ui->gamma, SIGNAL(clicked()), SIGNAL(changed()) );
  connect( ui->desaturate, SIGNAL(clicked()), SIGNAL(changed()) );
  connect( ui->colorize, SIGNAL(clicked()), SIGNAL(changed()) );
  connect( ui->semitransparent, SIGNAL(clicked()), SIGNAL(changed()) );
  connect( ui->bubblesmall, SIGNAL(clicked()), SIGNAL(changed()) );
  connect( ui->bubblelarge, SIGNAL(clicked()), SIGNAL(changed()) );
  connect( ui->bubbleno, SIGNAL(clicked()), SIGNAL(changed()) );
  connect( ui->defaultavatar, SIGNAL(clicked()), SIGNAL(changed()) );
  connect( ui->customavatar, SIGNAL(clicked()), SIGNAL(changed()) );
  connect( ui->pushButton1, SIGNAL(clicked()), SIGNAL(changed()) );
  connect( ui->defaultfunctionavatar, SIGNAL(clicked()), SIGNAL(changed()) );
  connect( ui->radioButton19, SIGNAL(clicked()), SIGNAL(changed()) );
  connect( ui->options, SIGNAL(returnPressed()), SIGNAL(changed()) );
  connect( ui->pushButton1, SIGNAL(clicked()), this, SLOT(openFileDialog()) );
  static const char *Programas[] =
    {"Konqueror", "Kontact", "Kopete", "Kaffeine", "AmaroK", "KControl", "Juk", 0
    };
  ui->comboBox1->insertStrList( Programas );
  ui->options->setEnabled(false);
  load( conf );
  ui->show();
}


nVIDIAConfig::~nVIDIAConfig()
{
  delete ui;
  delete c;
}

void nVIDIAConfig::openFileDialog()
{
  ui->lineEdit2->setText(QFileDialog::getOpenFileName(
                          c->readEntry("CustomAvatarPix", QString::null),
                        "Avatar Pixmaps(*.png)",
                        ui->pushButton1,
                        "open file dialog",
                        "Choose an Avatar Pixmap" ));
  ui->imagen->setPixmap(QPixmap(ui->lineEdit2->text()));
}
// Loads the configurable options from the kwinrc config file
// It is passed the open config from kwindecoration to improve efficiency
void nVIDIAConfig::load( KConfig* )
{
  register int shadowtype, bubbletype, ava, avafunc;
  QString textpos, coloroption, effects;
  c->setGroup("General");
  ui->showAppIcons->setChecked( c->readBoolEntry("ShowAppIcons", true) );
  ui->largeGrabBars->setChecked( c->readBoolEntry("LargeGrabBars", false) );
  ui->useShadowedText->setChecked( c->readBoolEntry("UseShadowedText", true) );
  ui->semitransparent->setChecked( c->readBoolEntry( "UseIconTransparent", false ) );
  ui->ShadowColor->setColor(c->readColorEntry("UseShadowedTextColor"));
  ui->effectcolor->setColor(c->readColorEntry("UseIconColor"));
  ui->effectvalue->setValue(c->readNumEntry("EffectValue", 50));
  textpos = c->readEntry("TextPosition", "left");
  shadowtype = c->readNumEntry("UseShadowedTextType", 1);
  coloroption = c->readEntry("UseColorOption", "system");
  effects = c->readEntry("Effect", "togray");
  bubbletype = c->readNumEntry("Bubble", 0);
  ui->lineEdit2->setText( c->readEntry("CustomAvatarPix", QString::null));
  ui->options->setText( c->readEntry( "AvatarFuncURL", "www.nvidia.com"));
  ui->comboBox1->setCurrentText( c->readEntry("AvatarFuncType", "Konqueror"));
  ava = c->readNumEntry("Avatar", 0);
  avafunc = c->readNumEntry("AvatarFunc", 0);
  ui->imagen->setPixmap(QPixmap(ui->lineEdit2->text()));
  if(ava == 0)
    ui->defaultavatar->setChecked( true );
  else
    ui->customavatar->setChecked( true );
  if(avafunc == 0)
    ui->defaultfunctionavatar->setChecked( true );  
  else
    ui->radioButton19->setChecked( true );
  if(bubbletype == 0)
    {
      ui->bubbleno->setChecked(true);
    }
  else if(bubbletype == 1)
    {
      ui->bubblesmall->setChecked(true);
    }
  else if(bubbletype == 2)
    {
      ui->bubblelarge->setChecked(true);
    }
  if(textpos == "left")
    {
      ui->left->setChecked(true);
    }
  else if(textpos == "center")
    {
      ui->center->setChecked(true);
    }
  else if(textpos == "right")
    {
      ui->right->setChecked(true);
    }
  if (coloroption == "system" )
    ui->systemcolor->setChecked(true);
  else
    ui->originalcolor->setChecked(true);
  if  (shadowtype == 1)
    {
      ui->shadow1->setChecked(true);
    }
  else if  (shadowtype == 2)
    {
      ui->shadow2->setChecked(true);
    }
  else if  (shadowtype == 3)
    {
      ui->shadow3->setChecked(true);
    }
  if (effects == "togray")
    {
      ui->togray->setChecked(true);
    }
  else if (effects == "colorize")
    {
      ui->colorize->setChecked(true);
    }
  else if (effects == "gamma")
    {
      ui->gamma->setChecked(true);
    }
  else if (effects == "desaturate")
    {
      ui->desaturate->setChecked(true);
    }
}


// Saves the configurable options to the kwinrc config file
void nVIDIAConfig::save( KConfig* )
{
  int bubble_type;
  if (ui->bubbleno->isChecked())
    {
      bubble_type = 0;
    }
  else if (ui->bubblesmall->isChecked())
    {
      bubble_type = 1;
    }
  else if (ui->bubblelarge->isChecked())
    {
      bubble_type = 2;
    }
  QString position;
  if (ui->left->isChecked())
    {
      position="left";
    }
  else if (ui->center->isChecked())
    {
      position ="center";
    }
  else if (ui->right->isChecked())
    {
      position ="right";
    }
  int shadowtype;
  if (ui->shadow1->isChecked())
    {
      shadowtype = 1;
    }
  else if (ui->shadow2->isChecked())
    {
      shadowtype = 2;
    }
  else if (ui->shadow3->isChecked())
    {
      shadowtype = 3;
    }
  QString coloropt;
  if(ui->systemcolor->isChecked())
    {
      coloropt = "system";
    }
  else if(ui->originalcolor->isChecked())
    {
      coloropt = "original";
    }
  QString effect;
  if(ui->togray->isChecked())
    {
      effect = "togray";
    }
  else if(ui->colorize->isChecked())
    {
      effect = "colorize";
    }
  else if(ui->gamma->isChecked())
    {
      effect = "gamma";
    }
  else if(ui->desaturate->isChecked())
    {
      effect = "desaturate";
    }
  register int avatar;
  if (ui->defaultavatar->isChecked())
    {
      avatar = 0;
    }
  else if(ui->customavatar->isChecked())
    {
      avatar = 1;
    }
  register int avatarfunc;
  if (ui->defaultfunctionavatar->isChecked())
    {
      avatarfunc = 0;
    }
  else if (ui->radioButton19->isChecked())
    {
      avatarfunc = 1;
    }
  QString Func, Type;  
  if(avatarfunc == 0)  {
  Type = "Konqueror";
  Func = "www.nvidia.com";
  }else
  {
  Type = ui->comboBox1->currentText();
  Func = ui->options->text();
  }
  ui->imagen->setPixmap(QPixmap(ui->lineEdit2->text()));
  c->setGroup( "General" );
  c->writeEntry( "ShowAppIcons", ui->showAppIcons->isChecked() );
  c->writeEntry( "LargeGrabBars", ui->largeGrabBars->isChecked() );
  c->writeEntry( "TextPosition", position );
  c->writeEntry( "UseShadowedText", ui->useShadowedText->isChecked() );
  c->writeEntry( "UseShadowedTextColor", ui->ShadowColor->color() );
  c->writeEntry( "UseShadowedTextType", shadowtype );
  c->writeEntry( "UseIconTransparent", ui->semitransparent->isChecked() );
  c->writeEntry( "UseIconColor", ui->effectcolor->color() );
  c->writeEntry( "EffectValue", ui->effectvalue->value() );
  c->writeEntry( "Effect", effect );
  c->writeEntry( "UseColorOption", coloropt );
  c->writeEntry( "Bubble", bubble_type );
  c->writeEntry( "Avatar", avatar);
  c->writeEntry( "AvatarFunc", avatarfunc);
  c->writeEntry( "CustomAvatarPix", ui->lineEdit2->text());
  c->writeEntry( "AvatarFuncType", Type );
  c->writeEntry( "AvatarFuncURL", Func);
  c->sync();
}

// Sets UI widget defaults which must correspond to style defaults
void nVIDIAConfig::defaults()
{
  ui->showAppIcons->setChecked( true );
  ui->useShadowedText->setChecked( true );
  ui->largeGrabBars->setChecked( false );
  ui->left->setChecked(true);
  ui->shadow2->setChecked(true);
  ui->originalcolor->setChecked(true);
  ui->effectvalue->setValue(50);
  ui->togray->setChecked(true);
  ui->ShadowColor->setColor(QColor(213, 213, 221));
  ui->effectcolor->setColor(QColor(0, 0, 0));
  ui->bubbleno->setChecked( true );
  ui->defaultavatar->setChecked( true );
  ui->defaultfunctionavatar->setChecked( true );
  emit changed();
}

void nVIDIAConfig::EnableURL(const QString &compa)
{
	if (compa == "Konqueror"){
	ui->options->setEnabled(true);
	}
	else
	ui->options->setEnabled(false);
  ui->imagen->setPixmap(QPixmap(ui->lineEdit2->text()));
}

// vim: set noet ts=4 sw=4: