/*
 * $Id: nvidia.cpp,v 1.0a 2004/01/09 18:20:29 giessl Exp $
 *
 * Keramik KWin client (version 0.8)
 *
 * Copyright (C) 2002 Fredrik H�lund <fredrik@kde.org>
 *
 * nVIDIA KWin client (version 1.0a)
 *
 * Copyright (C) 2004 Jose Rafael Castillo <jrch99@cantv.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the license, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <kconfig.h>
#include <klocale.h>
#include <kiconeffect.h>
#include <qcursor.h>
#include <qpainter.h>
#include <qlayout.h>
#include <qbitmap.h>
#include <qstyle.h>
#include <qtooltip.h>
#include <qwidget.h>
#include <qlabel.h>
#include <kprocess.h>
#include <X11/Xlib.h>

#include "nvidia.h"
#include "nvidia.moc"
// -------------------------------------------------------------------------------------------
namespace nVIDIA
  {
  const int buttonMargin     =  10;  // Margin between the window edge and the buttons//make a struct with this options ---BEGIN
  const int buttonSpacing    =  3;  // Spacing between the titlebar buttons
  const int iconSpacing      =  4;  // Spacing between the icon and the text label
  const int iconAppSpacing      =  1;  // Spacing between the icon and the border
  const int textSpacing      =  1;  // Spacing between the text and appicon
  // Default button layout
  const char default_left[]  = "";
  const char default_right[] = "HIAX";
  nVIDIAHandler *clientHandler = NULL;
  bool nvidia_initialized = false; //FIN
  // -------------------------------------------------------------------------------------------
  nVIDIAHandler::nVIDIAHandler()
  {
    for ( register int i = 0; i < NumTiles; i++ )
      {
        activeTiles[i]   = NULL;
        inactiveTiles[i] = NULL;
      }
    for ( register int b = 0; b < NumButton; b++ )
      {
        activeButtons[b]   = NULL;
        inactiveButtons[b] = NULL;
      }
    for ( register int e = 0; e < NumButtonDecos; e++ )
      {
        activebuttonDecos[e]   = NULL;
        inactivebuttonDecos[e] = NULL;
      }
    imageDb = nVIDIAImageDb::instance();
    readConfig();
    createPixmaps();


    nvidia_initialized = true;
  }

  nVIDIAHandler::~nVIDIAHandler()
  {
    nvidia_initialized = false;
    destroyPixmaps();
    nVIDIAImageDb::release();
    imageDb = NULL;
    clientHandler = NULL;
  }

  inline void nVIDIAHandler::createPixmaps() //----verify sizes
  {
    register int heightOffset;
    register int widthOffset;
    switch(options()->preferredBorderSize(this))
      {
      case BorderLarge:
        widthOffset = 4;
        heightOffset = 0;
        break;
      case BorderVeryLarge:
        widthOffset = 8;
        heightOffset = 0;
        break;     
      case BorderTiny:
      case BorderNormal:
      default:
        widthOffset = 0;
        heightOffset = 0;
      }//end
    //+revisar
    register int fontHeight = QFontMetrics(options()->font(true)).height();
    if (fontHeight > heightOffset + 20)
      heightOffset = fontHeight - 20;
    QString size = (heightOffset < 8) ? "" : (heightOffset < 20) ? "-large" : "-huge";
    QColor titleColor, captionColor, buttonColor;
    //+
    // Active tiles
    // -------------------------------------------------------------------------
    captionColor = KDecoration::options()->color( ColorTitleBar,   true );
    titleColor   = KDecoration::options()->color( ColorTitleBlend, true );
    // Load the titlebar corners.
    activeTiles[ TitleLeft ]  = loadPixmap( "titlebar-left-act");//,  titleColor );
#ifdef RIGHTRISTATE

    activeTiles[ TitleRight1 ] = loadPixmap( "titlebar-right1-act");//, titleColor );
    activeTiles[ TitleRight2 ] = loadPixmap( "titlebar-right2-act");//, titleColor );
    activeTiles[ TitleRight3 ] = loadPixmap( "titlebar-right3-act");//, titleColor );
#else

    activeTiles[ TitleRight ] = loadPixmap( "titlebar-right-act");//, titleColor );
#endif

    activeTiles[ TitleCenter ] = loadPixmap( "titlebar-center-act");
    // Load the left & right border pixmaps
    activeTiles[ BorderLeft ]  = loadPixmap( "border-left");//,  titleColor );
    activeTiles[ BorderRight ] = loadPixmap( "border-right");//, titleColor );
    // Load the bottom grabbar pixmaps
    if ( Settings_Param.largeGrabBars )
      {
        activeTiles[ GrabBarLeft ]   = loadPixmap( "grabbar-left");//,   titleColor );
        activeTiles[ GrabBarRight ]  = loadPixmap( "grabbar-right");//,  titleColor );
        activeTiles[ GrabBarCenter ] = loadPixmap( "grabbar-center");//, titleColor );
      }
    else
      {
        activeTiles[ GrabBarLeft ]   = loadPixmap( "bottom-left");//,   titleColor );
        activeTiles[ GrabBarRight ]  = loadPixmap( "bottom-right");//,  titleColor );
        activeTiles[ GrabBarCenter ] = loadPixmap( "bottom-center");//, titleColor );
      }
    // Inactive tiles
    // -------------------------------------------------------------------------
    captionColor = KDecoration::options()->color( ColorTitleBar,   false );
    titleColor   = KDecoration::options()->color( ColorTitleBlend, false );
    inactiveTiles[ TitleLeft ]  = loadPixmap( "titlebar-left-dis");//,  titleColor );
#ifdef RIGHTRISTATE

    inactiveTiles[ TitleRight1 ] = loadPixmap( "titlebar-right1-dis");//, titleColor );
    inactiveTiles[ TitleRight2 ] = loadPixmap( "titlebar-right2-dis");//, titleColor );
    inactiveTiles[ TitleRight3 ] = loadPixmap( "titlebar-right3-dis");//, titleColor );
#else

    inactiveTiles[ TitleRight ] = loadPixmap( "titlebar-right-dis");//, titleColor );
#endif

    inactiveTiles[ TitleCenter ] = loadPixmap( "titlebar-center-dis");
    inactiveTiles[ BorderLeft ]  = loadPixmap( "border-left");//,  titleColor );
    inactiveTiles[ BorderRight ] = loadPixmap( "border-right");//, titleColor );
    if ( Settings_Param.largeGrabBars )
      {
        inactiveTiles[ GrabBarLeft ]   = loadPixmap( "grabbar-left");//,   titleColor );
        inactiveTiles[ GrabBarRight ]  = loadPixmap( "grabbar-right");//,  titleColor );
        inactiveTiles[ GrabBarCenter ] = loadPixmap( "grabbar-center");//, titleColor );
      }
    else
      {
        inactiveTiles[ GrabBarLeft ]   = loadPixmap( "bottom-left");//,   titleColor );
        inactiveTiles[ GrabBarRight ]  = loadPixmap( "bottom-right");//,  titleColor );
        inactiveTiles[ GrabBarCenter ] = loadPixmap( "bottom-center");//, titleColor );
      }
    //active Buttons
    // -------------------------------------------------------------------------
    // buttonColor  =  KDecoration::options()->color( ButtonBg, true );
    activeButtons[ ButtonRound ] = loadPixmap("titlebutton-round-act"+size);
    activeButtons[ ButtonSquare] = loadPixmap("titlebutton-square-act"+size);
    //inactive Buttons
    // -------------------------------------------------------------------------
    //  buttonColor  =  KDecoration::options()->color( ButtonBg, false );
    inactiveButtons[ ButtonRound ] = loadPixmap("titlebutton-round-dis"+size);
    inactiveButtons[ ButtonSquare ] = loadPixmap("titlebutton-square-dis"+size);
    //active Buttons deco
    // -------------------------------------------------------------------------
    //  buttonColor  =  KDecoration::options()->color( ButtonBg, false );
#ifndef CUSTOMBUTTONS    
    activebuttonDecos[Menu] = loadPixmap("menu-act");
    activebuttonDecos[OnAllDesktops] = loadPixmap("alldesktop-act");
    activebuttonDecos[NotOnAllDesktops] = loadPixmap("noalldesktop-act");
#endif    
    activebuttonDecos[Help] = loadPixmap("help-act");
    activebuttonDecos[Minimize] = loadPixmap("minimize-act");
    activebuttonDecos[Maximize] = loadPixmap("maximize-act");
    activebuttonDecos[Restore] = loadPixmap("restore-act");
    activebuttonDecos[Close] = loadPixmap("close-act");
    //inactive Buttons deco
    // -------------------------------------------------------------------------
    //  buttonColor  =  KDecoration::options()->color( ButtonBg, false );
#ifndef CUSTOMBUTTONS        
    inactivebuttonDecos[Menu] = loadPixmap("menu-dis");
    inactivebuttonDecos[OnAllDesktops] = loadPixmap("alldesktop-dis");
    inactivebuttonDecos[NotOnAllDesktops] = loadPixmap("noalldesktop-dis");
#endif    
    inactivebuttonDecos[Help] = loadPixmap("help-dis");
    inactivebuttonDecos[Minimize] = loadPixmap("minimize-dis");
    inactivebuttonDecos[Maximize] = loadPixmap("maximize-dis");
    inactivebuttonDecos[Restore] = loadPixmap("restore-dis");
    inactivebuttonDecos[Close] = loadPixmap("close-dis");
    // Prepare the tiles for use
    // -------------------------------------------------------------------------
    if ( QApplication::reverseLayout() )
      {
        // Fix lighting
        if(Settings_Param.bubbletype != 0)
          {
            flip( activeTiles[CaptionSmallLeft], activeTiles[CaptionSmallRight] );
            flip( inactiveTiles[CaptionSmallLeft], inactiveTiles[CaptionSmallRight] );
            flip( activeTiles[CaptionLargeLeft], activeTiles[CaptionLargeRight] );
          }
        flip( activeTiles[TitleCenter]);
        flip( inactiveTiles[TitleCenter]);
#ifdef RIGHTRISTATE

        //flip( activeTiles[TitleLeft], activeTiles[nVIDIAClient::ButtonsLeft] );
        // flip( inactiveTiles[TitleLeft], inactiveTiles[nVIDIAClient::ButtonsLeft] );
#else

        flip( activeTiles[TitleLeft], activeTiles[TitleRight] );
        flip( inactiveTiles[TitleLeft], inactiveTiles[TitleRight] );
#endif

        flip( activeTiles[BorderLeft], activeTiles[BorderRight] );
        flip( inactiveTiles[BorderLeft], inactiveTiles[BorderRight] );
        flip( activeTiles[GrabBarLeft], activeTiles[GrabBarRight] );
        flip( inactiveTiles[GrabBarLeft], inactiveTiles[GrabBarRight] );
        flip( activeButtons[ ButtonRound ], activeButtons[ ButtonSquare ] );
        flip( inactiveButtons[ ButtonRound ], inactiveButtons[ ButtonSquare ] );
      }

    // Pretile the center & border tiles for optimal performance
    if(Settings_Param.bubbletype != 0)
      {
        pretile( activeTiles[ CaptionSmallCenter ], 64, Qt::Horizontal );
        pretile( activeTiles[ CaptionLargeCenter ], 64, Qt::Horizontal );
        pretile( inactiveTiles[ CaptionSmallCenter ], 64, Qt::Horizontal );
      }
    pretile( activeButtons[ ButtonRound ], 128, Qt::Horizontal );
    pretile( activeButtons[ ButtonSquare ], 128, Qt::Horizontal );
    pretile( activeTiles[ TitleCenter ], 64, Qt::Horizontal );
    pretile( activeTiles[ GrabBarCenter ], 128, Qt::Horizontal );
    pretile( activeTiles[ BorderLeft ], 128, Qt::Vertical );
    pretile( activeTiles[ BorderRight ], 128, Qt::Vertical );
    pretile( inactiveTiles[ TitleCenter ], 64, Qt::Horizontal );
    pretile( inactiveTiles[ GrabBarCenter ], 128, Qt::Horizontal );
    pretile( inactiveTiles[ BorderLeft ], 128, Qt::Vertical );
    pretile( inactiveTiles[ BorderRight ], 128, Qt::Vertical );
    pretile( inactiveButtons[ ButtonRound ], 128, Qt::Horizontal );
    pretile( inactiveButtons[ ButtonSquare ] , 128, Qt::Horizontal );
    if (heightOffset > 0)
      {
        addHeight (heightOffset, activeTiles[TitleLeft]);
        addHeight (heightOffset, activeTiles[TitleCenter]);
#ifdef RIGHTRISTATE

        //        addHeight (heightOffset, inactiveTiles[nVIDIAClient::ButtonsLeft]);
#else

        addHeight (heightOffset, activeTiles[TitleRight]);
#endif

        if(Settings_Param.bubbletype != 0)
          {
            addHeight (heightOffset, activeTiles[CaptionSmallLeft]);
            addHeight (heightOffset, activeTiles[CaptionSmallCenter]);
            addHeight (heightOffset, activeTiles[CaptionSmallRight]);
            addHeight (heightOffset, activeTiles[CaptionLargeLeft]);
            addHeight (heightOffset, activeTiles[CaptionLargeCenter]);
            addHeight (heightOffset, activeTiles[CaptionLargeRight]);
            addHeight (heightOffset, inactiveTiles[CaptionSmallLeft]);
            addHeight (heightOffset, inactiveTiles[CaptionSmallCenter]);
            addHeight (heightOffset, inactiveTiles[CaptionSmallRight]);
          }
        addHeight (heightOffset, inactiveTiles[TitleLeft]);
        addHeight (heightOffset, inactiveTiles[TitleCenter]);
#ifdef RIGHTRISTATE

        //        addHeight (heightOffset, inactiveTiles[nVIDIAClient::ButtonsLeft]);
#else

        addHeight (heightOffset, inactiveTiles[TitleRight]);
#endif

      }

    if (widthOffset > 0)
      {
        addWidth (widthOffset, activeTiles[BorderLeft], true, activeTiles[GrabBarCenter]);
        addWidth (widthOffset, activeTiles[BorderRight], false, activeTiles[GrabBarCenter]);
        addWidth (widthOffset, inactiveTiles[BorderLeft], true, inactiveTiles[GrabBarCenter]);
        addWidth (widthOffset, inactiveTiles[BorderRight], false, inactiveTiles[GrabBarCenter]);

        if (Settings_Param.largeGrabBars)
          widthOffset = widthOffset*3/2;

        addHeight (widthOffset, activeTiles[GrabBarLeft]);
        addHeight (widthOffset, activeTiles[GrabBarCenter]);
        addHeight (widthOffset, activeTiles[GrabBarRight]);
        addHeight (widthOffset, inactiveTiles[GrabBarLeft]);
        addHeight (widthOffset, inactiveTiles[GrabBarCenter]);
        addHeight (widthOffset, inactiveTiles[GrabBarRight]);
      }
  }

  inline void nVIDIAHandler::destroyPixmaps()
  {
    for ( register int i = 0; i < NumTiles; i++ )
      {
        delete activeTiles[i];
        delete inactiveTiles[i];
        activeTiles[i] = NULL;
        inactiveTiles[i] = NULL;
      }
    for ( register int a = 0; a < NumButton; a++ )
      {
        delete activeButtons[a];
        delete inactiveButtons[a];
        activeButtons[a] = NULL;
        inactiveButtons[a] = NULL;
      }
    for ( register int c = 0; c < NumButtonDecos; c++)
      {
        delete activebuttonDecos[c];
        delete inactivebuttonDecos[c];
        activebuttonDecos[c] = NULL;
        inactivebuttonDecos[c] = NULL;
      }
  }

  void nVIDIAHandler::addWidth (register int width, QPixmap *&pix, bool left, QPixmap *bottomPix)
  {
    register int w = pix->width()+width;
    register int h = pix->height();
    QPixmap *tmp = new QPixmap (w, h);
    tmp->fill ();
    QPainter p;
    p.begin (tmp);
    for (register int i = 0; i < h; i++)
      p.drawPixmap (0, i, *bottomPix, i%2, 0, w,1);
    if (left)
      p.drawPixmap(0, 0, *pix);
    else
      p.drawPixmap(width, 0, *pix);
    p.end();
    delete pix;
    pix = tmp;
  }

  void nVIDIAHandler::addHeight (register int height, QPixmap *&pix)
  {
    register int w = pix->width();
    register int h = pix->height()+height;
    QPixmap *tmp = new QPixmap (w, h);
    QPainter p;
    p.begin (tmp);
    if (pix->height() > 10)
      {
        p.drawPixmap(0, 0, *pix, 0, 0, w, 11);
        for (register int i = 0; i < height; i+=2)
          p.drawPixmap(0, 11+i, *pix, 0, 11, w, 2);
        p.drawPixmap(0, 11+height, *pix, 0, 11, w, -1);
      }
    else
      {
        register int lines  = h-3;
        register int factor = pix->height()-3;
        for (register int i = 0; i < lines; i++)
          p.drawPixmap(0, i, *pix, 0, i*factor/lines, w, 1);
        p.drawPixmap(0, lines, *pix, 0, factor, w, 3);
      }
    p.end();
    delete pix;
    pix = tmp;
  }

  void nVIDIAHandler::flip( QPixmap *&pix1, QPixmap *&pix2 )
  {
    // Flip the pixmaps horizontally
    QPixmap *tmp = new QPixmap( pix1->xForm( QWMatrix(-1,0,0,1,pix1->width(),0) ) );
    delete pix1;
    pix1 = new QPixmap( pix2->xForm( QWMatrix(-1,0,0,1,pix2->width(),0) ) );
    delete pix2;
    pix2 = tmp;
  }

  void nVIDIAHandler::flip( QPixmap *&pix )
  {
    // Flip the pixmap horizontally
    QPixmap *tmp = new QPixmap( pix->xForm( QWMatrix(-1,0,0,1,pix->width(),0) ) );
    delete pix;
    pix = tmp;
  }

  void nVIDIAHandler::pretile( QPixmap *&pix, register int size, Qt::Orientation dir )
  {
    QPixmap *newpix;
    QPainter p;
    if ( dir == Qt::Horizontal )
      newpix = new QPixmap( size, pix->height() );
    else
      newpix = new QPixmap( pix->width(), size );
    p.begin( newpix );
    p.drawTiledPixmap( newpix->rect(), *pix ) ;
    p.end();
    delete pix;
    pix = newpix;
  }

  inline void nVIDIAHandler::readConfig()//rehacer
  {
    KConfig *c = new KConfig( "kwinnvidiarc" );
    c->setGroup( "General" );
    Settings_Param.showIcons = c->readBoolEntry( "ShowAppIcons", true );
    Settings_Param.shadowedText = c->readBoolEntry( "UseShadowedText", true );
    Settings_Param.bubbletype = c->readNumEntry( "Bubble", 0 );
    Settings_Param.largeGrabBars = c->readBoolEntry( "LargeGrabBars", false );
    Settings_Param.textpos = c->readEntry("TextPosition", "left");
    Settings_Param.shadowtype = c->readNumEntry("UseShadowedTextType", 1);
    Settings_Param.coloroption = c->readEntry("UseColorOption", "system");
    Settings_Param.effects = c->readEntry("Effect", "togray");
    Settings_Param.ShadowColor = c->readColorEntry("UseShadowedTextColor");
    Settings_Param.Transparency = c->readBoolEntry( "UseIconTransparent", false );
    Settings_Param.IconEffectColor = c->readColorEntry("UseIconColor");
    Settings_Param.effectvalues = c->readNumEntry( "EffectValue", 50 );
#ifdef AVATARENABLED

    Avatar_Params.avatarfuncURL = c->readEntry("AvatarFuncURL", "www.nvidia.com");
    Avatar_Params.avatarpix = c->readEntry("CustomAvatarPix", QString::null);
    Avatar_Params.avatarfunctype = c->readEntry("AvatarFuncType", "Konqueror");
    Avatar_Params.avatarfunc = c->readNumEntry("AvatarFunc", 0);
    Avatar_Params.avatar = c->readNumEntry("Avatar", 0);
#endif

    if (Settings_Param.bubbletype == 1)
      {
        Settings_Param.smallCaptionBubbles = true;
      }
    delete c;
  }

  inline QImage *nVIDIAHandler::loadImage( const QString &name)//, const QColor &col )
  {
    // if ( col.isValid() )
    //{
    QImage *img = new QImage( imageDb->image(name)->copy() );
    //  KIconEffect::colorize( *img, col, 1.0 );
    return img;
    // }
    //else
    // return new QImage( imageDb->image(name)->copy() );
  }

  inline QPixmap *nVIDIAHandler::loadPixmap( const QString &name)//, const QColor &col )
  {
    QImage *img = loadImage( name);//, col );
    QPixmap *pix = new QPixmap( *img );
    delete img;
    return pix;
  }

  inline bool nVIDIAHandler::reset( unsigned long changed )
  {
    nvidia_initialized = false;
    bool needHardReset  = false;
    bool pixmapsInvalid = false;
    // Re-read the config file
    readConfig();
    if ( changed & SettingBorder )
      {
        pixmapsInvalid = true;
        needHardReset = true;
      }
    if ( changed & SettingFont )
      {
        pixmapsInvalid = true;
        needHardReset = true;
      }
    // Check if the color scheme has changed
    if ( changed & SettingColors )
      {
        pixmapsInvalid = true;
      }
    // Check if button positions have changed

    if ( changed & SettingButtons )
      {
        needHardReset = true;
      }

    // Check if tooltips options have changed
    if ( changed & SettingTooltips )
      {
        needHardReset = true;
      }

    if ( !Settings_Param.largeGrabBars)
      {
        pixmapsInvalid = true;
        needHardReset = true;
      }

    // Do we need to recreate the pixmaps?
    if ( pixmapsInvalid )
      {
        destroyPixmaps();
        createPixmaps();
      }
    nvidia_initialized = true;
    // Do we need to "hit the wooden hammer" ?
    if ( !needHardReset )
      resetDecorations( changed );
    return needHardReset;
  }

  const QPixmap *nVIDIAHandler::tile( TilePixmap tilePix, bool active ) const
    {
      return ( active ? activeTiles[ tilePix ] : inactiveTiles[ tilePix ] );
    }

  const QPixmap *nVIDIAHandler::btile(TileButtons buttonPix, bool active) const
    {
      return (active ? activeButtons[ buttonPix ] : inactiveButtons[ buttonPix ]);
    }

  const QPixmap *nVIDIAHandler::decotile(ButtonDeco tiledeco, bool active) const
    {
      return (active ? activebuttonDecos[ tiledeco ] : inactivebuttonDecos[ tiledeco ]);
    }

  KDecoration* nVIDIAHandler::createDecoration( KDecorationBridge* bridge )
  {
    return new nVIDIAClient( bridge, this );
  }

  QValueList< nVIDIAHandler::BorderSize > nVIDIAHandler::borderSizes() const
    { // the list must be sorted
      return QValueList< BorderSize >() << BorderNormal << BorderLarge <<
             BorderVeryLarge;
    }
  // revisado-----------------------------------------------------------------------------------
  nVIDIAButton::nVIDIAButton( nVIDIAClient* c, const char *name, Button btn, const QString &tip, const int realizeBtns )
      : QButton( c->widget(), name ),
      client( c ), button( btn ), hover( false ), lastbutton( 0 )
  {
    realizeButtons = realizeBtns;
    bool active = client->isActive();
    QToolTip::add
      ( this, tip ); // FRAME
    setBackgroundMode( NoBackground );
    setCursor( arrowCursor );
    if (name == "menu" || name == "on_all_desktops" || name == "help" )
      {
        setAutoMask(true);
        register int size = clientHandler->btile( ButtonRound, active )->height();
        setFixedSize( size, size );
      }
    else
      {
        setAutoMask(false);
        register int size = clientHandler->btile( ButtonSquare, active )->height();
        setFixedSize( size, size );
      }
#ifndef CUSTOMBUTTONS      
    setToggleButton( (button == OnAllDesktopsButton) );
#endif
  }
#ifdef CUSTOMBUTTONS
nVIDIAButton::nVIDIAButton( nVIDIAClient* c, const char *name, Button btn, const QString &tip )
      : QButton( c->widget(), name ),
      client( c ), button( btn ), hover( false ), lastbutton( 0 )
  {
    bool active = client->isActive();
    QToolTip::add
      ( this, tip ); // FRAME
    setBackgroundMode( NoBackground );
    setCursor( arrowCursor );
    if (name == "menu" || name == "on_all_desktops" || name == "help" )
      {
        setAutoMask(true);
        register int size = clientHandler->btile( ButtonRound, active )->height();
        setFixedSize( size, size );
      }
    else
      {
        setAutoMask(false);
        register int size = clientHandler->btile( ButtonSquare, active )->height();
        setFixedSize( size, size );
      }
#ifndef CUSTOMBUTTONS            
    setToggleButton( (button == OnAllDesktopsButton) );
#endif    
  }
#endif
  nVIDIAButton::~nVIDIAButton()
  {
    // Empty.
  }

  void nVIDIAButton::enterEvent( QEvent *e )
  {
    QButton::enterEvent( e );
    hover = true;
    update();
  }

  void nVIDIAButton::leaveEvent( QEvent *e )
  {
    QButton::leaveEvent( e );
    hover = false;
    update();
  }

  void nVIDIAButton::mousePressEvent( QMouseEvent *e )
  {
    lastbutton = e->button();
    QMouseEvent me( e->type(), e->pos(), e->globalPos(), (e->button()&realizeButtons)?LeftButton:NoButton, e->state() );
    QButton::mousePressEvent( &me );
  }

  void nVIDIAButton::mouseReleaseEvent( QMouseEvent *e )
  {
    lastbutton = e->button();
    QMouseEvent me( e->type(), e->pos(), e->globalPos(), (e->button()&realizeButtons)?LeftButton:NoButton, e->state() );
    QButton::mouseReleaseEvent( &me );
  }

  inline void nVIDIAButton::drawButton( QPainter *p )
  {
    const QPixmap *deco;
    const QPixmap *pix;
    register int size;
    // Get the bevel from the client handler
#ifdef CUSTOMBUTTONS
    if ( button == HelpButton )
#else    
    if ( button == MenuButton || button == OnAllDesktopsButton || button == HelpButton )
#endif    
      {
        pix = clientHandler->btile( ButtonRound, client->isActive() );
        size = clientHandler->btile( ButtonRound, client->isActive() )->height();
      }
    else
      {
        pix = clientHandler->btile( ButtonSquare, client->isActive() );
        size = clientHandler->btile( ButtonSquare, client->isActive() )->height();
      }

    if ( isDown() )
      {
        // Pressed
#ifdef CUSTOMBUTTONS
    if ( button == HelpButton )
#else    
    if ( button == MenuButton || button == OnAllDesktopsButton || button == HelpButton )
#endif           
          {
            p->drawPixmap( QPoint(), *pix, QStyle::visualRect( QRect(2*size, 0, size, size), pix->rect() ) );
          }
        else
          {
            p->drawPixmap( QPoint(), *pix, QStyle::visualRect( QRect(2*size+iconSpacing, 0, size, size), pix->rect() ) );
            p->translate( QApplication::reverseLayout() ? -1 : 1,  1 );
          }
      }
    else if ( hover )
      // Mouse over
#ifdef CUSTOMBUTTONS
    if ( button == HelpButton )
#else    
    if ( button == MenuButton || button == OnAllDesktopsButton || button == HelpButton )
#endif    
        {
          p->drawPixmap( QPoint(), *pix, QStyle::visualRect( QRect(size, 0, size, size), pix->rect() ) );
        }
      else
        {
          p->drawPixmap( QPoint(), *pix, QStyle::visualRect( QRect(size+iconSpacing-2, 0, size, size), pix->rect() ) );
        }
    else
      // Normal
#ifdef CUSTOMBUTTONS
    if ( button == HelpButton )
#else    
    if ( button == MenuButton || button == OnAllDesktopsButton || button == HelpButton )
#endif    
        {
          p->drawPixmap( QPoint(), *pix, QStyle::visualRect( QRect(0, 0, size, size), pix->rect() ) );
        }
      else
        {
          p->drawPixmap( QPoint(), *pix, QStyle::visualRect( QRect(0, 0, size, size), pix->rect() ) );
        }
//ready
    // Draw the button deco on the bevel
    
    switch ( button )
      {
#ifndef CUSTOMBUTTONS      
      case MenuButton:
          deco = clientHandler->decotile(Menu, client->isActive());  
        break;

      case OnAllDesktopsButton:        
          deco = clientHandler->decotile( isOn() ? NotOnAllDesktops : OnAllDesktops, client->isActive() );  
        break;
#endif
      case HelpButton:        
          deco = clientHandler->decotile( Help, client->isActive() );  
        if ( QApplication::reverseLayout() )
          p->translate( 2, 0 );
        break;

      case MinButton:        
          deco = clientHandler->decotile( Minimize, client->isActive() );	  
        break;

      case MaxButton:       
          deco = clientHandler->decotile( client->maximizeMode() == nVIDIAClient::MaximizeFull ? Restore : Maximize, client->isActive() );  
        break;

      case CloseButton:       
          deco = clientHandler->decotile( Close, client->isActive() );	  
        break;
      default:
        deco = NULL;
      }
#ifdef CUSTOMBUTTONS
    if ( button == HelpButton )
#else    
    if ( button == MenuButton || button == OnAllDesktopsButton || button == HelpButton )
#endif    
      p->drawPixmap( 4, 3, *deco );
    else
      p->drawPixmap( (size-clientHandler->decotile( Close, client->isActive() )->width()+1)/2, ((size-clientHandler->decotile( Close, client->isActive() )->height())/2), *deco );
  }

  inline void nVIDIAButton::updateMask()
  {
    bool active =client->isActive();//check
    QRegion mask( 0, 0, clientHandler->btile( ButtonRound, active )->width(), clientHandler->btile( ButtonRound, active )->height() );
    // Remove Round button corners
    mask -= QRegion( 0, 0, 3, 1 );
    mask -= QRegion( 0, 1, 2, 1 );
    mask -= QRegion( 0, 2, 1, 1 );
    mask -= QRegion( 0, 11, 1, 1 );
    mask -= QRegion( 0, 12, 2, 1 );
    mask -= QRegion( 0, 13, 3, 1 );
    mask -= QRegion( 11, 0, 6, 1 );
    mask -= QRegion( 12, 1, 4, 1 );
    mask -= QRegion( 13, 2, 2, 1 );
    mask -= QRegion( 13, 11, 2, 1 );
    mask -= QRegion( 12, 12, 4, 1 );
    mask -= QRegion( 11, 13, 6, 1 );
    mask -= QRegion( 25, 0, 6, 1 );
    mask -= QRegion( 26, 1, 4, 1 );
    mask -= QRegion( 27, 2, 2, 1 );
    mask -= QRegion( 27, 11, 2, 1 );
    mask -= QRegion( 26, 12, 4, 1 );
    mask -= QRegion( 25, 13, 6, 1 );
    mask -= QRegion( 38, 0, 3, 1 );
    mask -= QRegion( 40, 1, 2, 1 );
    mask -= QRegion( 41, 2, 1, 2 );
    mask -= QRegion( 41, 11, 1, 1 );
    mask -= QRegion( 40, 12, 2, 1 );
    mask -= QRegion( 39, 13, 3, 1 );
    setMask(mask);
  }

  //--------------------------------------------------------------------------------------------
#ifdef AVATARENABLED
  nVIDIAAvatarButton::nVIDIAAvatarButton(nVIDIAClient* c, const char *name, const QString &tip) : QButton( c->widget(), name ), client( c ), over( false )
  {
    setBackgroundMode( NoBackground );
    setCursor( QCursor(Qt::PointingHandCursor) );
    if(Avatar_Params.avatar == 0)
      setFixedSize( 24, 16 );
    else if(Avatar_Params.avatar == 1)
      setFixedSize( 22, 22 );
    setAutoMask(true);
    QToolTip::add
      ( this, tip );
  }

  nVIDIAAvatarButton::~nVIDIAAvatarButton()
{}

  inline void nVIDIAAvatarButton::updateMask()
  {
    QImage img0 = LoadAvatar().copy();
    img0.setAlphaBuffer(true);
    QBitmap Mask;
    if(img0.hasAlphaBuffer())
    Mask = img0.createAlphaMask();
    else
    Mask =img0.createHeuristicMask();    
    setMask(Mask);
  }

  inline QImage nVIDIAAvatarButton::LoadAvatar()
  {
    QImage temp;
    if(Avatar_Params.avatar == 1)
      {
        temp = FastScaleImage(Avatar_Params.avatarpix, 22, 22);
      }
    else if(Avatar_Params.avatar == 0)
      {
        nVIDIAImageDb *imageDb;
        imageDb = nVIDIAImageDb::instance();
        temp = imageDb->image("avatar")->copy();
      }
    return temp;
  }

  inline QImage nVIDIAAvatarButton::FastScaleImage(const QString &imagen, int w, int h)
  {
    QImage img;
    img.load(imagen);
    QImage imgtemp = img.smoothScale(w, h, QImage::ScaleFree);
    return imgtemp;
  }

  void nVIDIAAvatarButton::enterEvent( QEvent * e)
  {
    QButton::enterEvent( e );
    over = true;
    update();
  }

  void nVIDIAAvatarButton::leaveEvent( QEvent * e )
  {
    QButton::leaveEvent( e );
    over = false;
    update();
  }

  inline void nVIDIAAvatarButton::drawButton( QPainter * p)
  {
    bool active = client->isActive();
    QImage img = LoadAvatar();
    if(isDown())
      {
        if(!active)
          {
            KIconEffect::toGray(img, 100);
          }
        else
          {
            KIconEffect:: toGamma(img, 0);
          }
        p->drawImage(QPoint(), img);
      }
    else if (over)
      {
        if(!active)
          {
            KIconEffect::semiTransparent(img);
            KIconEffect::toGray(img, 100);
          }
        else
          {
            KIconEffect::semiTransparent(img);
          }
        p->drawImage(QPoint(), img);
      }
    else
      {
        if(!active)
          {
            KIconEffect::toGray(img, 100);
          }
        else
          img = img;
        p->drawImage(QPoint(), img);
      }
  }

  // revisado-----------------------------------------------------------------------------------
#endif

  nVIDIAClient::nVIDIAClient( KDecorationBridge* bridge, KDecorationFactory* factory )
      : KDecoration( bridge, factory ),
      activeIcon( NULL ), inactiveIcon( NULL ), maskDirty( true ), captionBufferDirty( true ), iconBufferDirty( true )
  {
    captionBufferDirty = true;
    iconBufferDirty = true;
    Settings_Param.maxe=isMaximizable();
    Settings_Param.mine=isMinimizable();
    Settings_Param.closee=isCloseable();
  }

  void nVIDIAClient::init()
  {
    createMainWidget( WStaticContents | WResizeNoErase | WRepaintNoErase );
    widget()->installEventFilter( this );
#ifdef RIGHTRISTATE

    ButtonsLeft = ButtonsOnLeft();
#endif
    // Minimize flicker
    widget()->setBackgroundMode( NoBackground );

    for ( register int i=0; i < NumButtons; i++ )
      button[i] = NULL;

    createLayout();
  }

  inline void nVIDIAClient::createLayout()
  {
    QVBoxLayout *mainLayout   = new QVBoxLayout( widget() );
    QBoxLayout *titleLayout   = new QBoxLayout( 0, QBoxLayout::LeftToRight, 0, 0, 0 );
    QHBoxLayout *windowLayout = new QHBoxLayout();
    register int grabBarHeight = clientHandler->grabBarHeight();
    if(Settings_Param.bubbletype != 0)
      Settings_Param.topSpacing = (clientHandler->tile(CaptionLargeCenter, isActive())-clientHandler->tile(TitleCenter, isActive()));
    else
      Settings_Param.topSpacing = -2;
    register int leftBorderWidth    = clientHandler->tile( BorderLeft, isActive() )->width();
    register int rightBorderWidth   = clientHandler->tile( BorderRight, isActive() )->width();
    topSpacer = new QSpacerItem( 10, Settings_Param.topSpacing,
                                 QSizePolicy::Expanding, QSizePolicy::Minimum );

    mainLayout->addItem( topSpacer );

    mainLayout->addLayout( titleLayout );     // Titlebar
    mainLayout->addLayout( windowLayout, 1 ); // Left border + window + right border
    mainLayout->addSpacing( grabBarHeight );  // Bottom grab bar

    titleLayout->setSpacing( buttonSpacing );

    titleLayout->addSpacing( buttonMargin );      // Left button margin
    addButtons( titleLayout, options()->customButtonPositions() ?
                options()->titleButtonsLeft() : QString(default_left) );
    addAvatar();

    titlebar = new QSpacerItem( 10, clientHandler->titleBarHeight(false) - Settings_Param.topSpacing, QSizePolicy::Expanding, QSizePolicy::Minimum);
    titleLayout->addItem( titlebar );

    titleLayout->addSpacing( buttonSpacing );
    addButtons( titleLayout, options()->customButtonPositions() ?
                options()->titleButtonsRight() : QString(default_right) );
    titleLayout->addSpacing( buttonMargin - 1 );  // Right button margin

    windowLayout->addSpacing( leftBorderWidth );                // Left border
    if( isPreview())
      windowLayout->addWidget( new QLabel( i18n( "<center><b><u>Advanced Kwin Client Engine for KDE 3.2</u>\n nVIDIA Theme \n by Jose Rafael Castillo Huggins</b></center>" ), widget()));
    else
      windowLayout->addItem( new QSpacerItem( 0, 0 )); //no widget in the middle
    windowLayout->addSpacing( rightBorderWidth );                // Right border

  }

  nVIDIAClient::~nVIDIAClient()
  {
    delete activeIcon;
    delete inactiveIcon;
    activeIcon = inactiveIcon = NULL;
  }

  void nVIDIAClient::reset( unsigned long )
  {
    if(Settings_Param.bubbletype != 0)
      {
        if ( !maximizedVertical() )
          {
            if (Settings_Param.bubbletype !=0 )
              topSpacer->changeSize( 10, clientHandler->tile(CaptionLargeCenter, isActive())->height()-clientHandler->tile(TitleCenter, isActive())->height(), QSizePolicy::Expanding, QSizePolicy::Minimum );
            else
              topSpacer->changeSize( 10, 0, QSizePolicy::Expanding, QSizePolicy::Minimum );
            maskDirty = true;
            widget()->layout()->activate();
            // Compensate for the titlebar size change
            // FRAME
            widget()->setGeometry( widget()->x(), widget()->y() - 3, width(), height() + 3 );
          }
      }
    maskDirty = true;
    captionBufferDirty = true;
    iconBufferDirty = true;
    // Only repaint the window if it's visible
    // (i.e. not minimized and on the current desktop)
    if ( widget()->isVisible() )
      {
        widget()->update();
#ifdef AVATARENABLED

        btt->update();
#endif
#ifdef RIGHTRISTATE

        ButtonsLeft = ButtonsOnLeft();
#endif
        for ( register int i = 0; i < NumButtons; i++ )
          if ( button[i] )
            button[i]->update();
      }
  }

#ifdef AVATARENABLED
  void nVIDIAClient::addAvatar()
  {
    btt = new nVIDIAAvatarButton( this, "Kwin Functional Button", i18n("Kwin Functional Button") );
    connect(btt, SIGNAL(pressed()), this, SLOT(AvatarAction()));
    if(Avatar_Params.avatar == 0)
    btt->setGeometry(5, 5, this->width(), this->height());
    else if(Avatar_Params.avatar == 1)
    btt->setGeometry(((clientHandler->tile(TitleLeft, isActive())->width()-btt->width())/2)-2, (clientHandler->tile(TitleLeft, isActive())->height()-btt->height())/2, this->width(), this->height());
  }
#endif

  void nVIDIAClient::addButtons( QBoxLayout *layout, const QString &s )
  {
    Settings_Param.btnposflag = LeftButton|RightButton;
    bool active = isActive();
#ifdef CUSTOMBUTTONS
    // Minimize button
    for ( register uint i=0; i < s.length(); i++ )
      {
        switch ( s[i].latin1() )
          {
          case 'I' :
            if ( !button[MinButton] && isMinimizable() )
              {
                button[MinButton] = new nVIDIAButton( this, "minimize", MinButton, i18n("Minimize"), LeftButton|MidButton|RightButton  );
                connect( button[MinButton], SIGNAL( clicked() ), SLOT( minimize() ) );
                layout->addWidget( button[MinButton] );
              }
            break;

            // Maximize button
          case 'A' :
            if ( !button[MaxButton] && isMaximizable() )
              {
                button[MaxButton] = new nVIDIAButton( this, "maximize", MaxButton, i18n("Maximize"), LeftButton|MidButton|RightButton );
                connect( button[MaxButton], SIGNAL( clicked() ), SLOT( slotMaximize() ) );
                layout->addWidget( button[MaxButton] );
              }
            break;

            // Close button
          case 'X' :
            if ( !button[CloseButton] && isCloseable() )
              {
                button[CloseButton] = new nVIDIAButton( this, "close", CloseButton, i18n("Close"), LeftButton|MidButton|RightButton  );
                connect( button[CloseButton], SIGNAL( clicked() ), SLOT( closeWindow() ) );
                layout->addWidget( button[CloseButton] );
              }
            break;

            // Additional spacing
          case '_' :
            layout->addSpacing( buttonSpacing );
            break;
	              
	  case 'H' :
#ifdef RIGHTRISTATE
        if (!button[HelpButton] && providesContextHelp() )
          {
            switch (ButtonsLeft)
              {
              case 3:
                button[HelpButton] = new nVIDIAButton( this, "help", HelpButton, i18n("Help"), LeftButton|MidButton|RightButton  );
                button[HelpButton]->setGeometry((width() - (clientHandler->tile( TitleRight1, active )->width()-clientHandler->btile( ButtonRound, active )->width()))+5, 7, button[HelpButton]->width(), button[HelpButton]->height());
                break;
              case 4:
                button[HelpButton] = new nVIDIAButton( this, "help", HelpButton, i18n("Help"), LeftButton|MidButton|RightButton  );
                button[HelpButton]->setGeometry((width() - (clientHandler->tile( TitleRight2, active )->width()-clientHandler->btile( ButtonRound, active )->width()))+5, 7, button[HelpButton]->width(), button[HelpButton]->height());
                break;
              case 5:
                button[HelpButton] = new nVIDIAButton( this, "help", HelpButton, i18n("Help"), LeftButton|MidButton|RightButton  );
                button[HelpButton]->setGeometry((width() - (clientHandler->tile( TitleRight3, active )->width()-clientHandler->btile( ButtonRound, active )->width()))+5, 7, button[HelpButton]->width(), button[HelpButton]->height());
                break;
              }
          }
#else

        if (!button[HelpButton] && providesContextHelp() )
          {
            button[HelpButton] = new nVIDIAButton( this, "help", HelpButton, i18n("Help"), LeftButton|MidButton|RightButton  );
            button[HelpButton]->setGeometry((width()-(clientHandler->tile( TitleRight, active )->width()-clientHandler->btile( ButtonRound, active )->width()))+5, 7, button[HelpButton]->width(), button[HelpButton]->height());
          }
#endif
	break;
	}
	connect( button[HelpButton], SIGNAL( clicked() ), SLOT( showContextHelp() ) );
}
  #else

    for ( register uint i=0; i < s.length(); i++ )
      {
        switch ( s[i].latin1() )
          {
            // Menu button
          case 'M' :
            if ( !button[MenuButton] )
              {
                button[MenuButton] = new nVIDIAButton( this, "menu", MenuButton, i18n("Menu"), Settings_Param.btnposflag );
                connect( button[MenuButton], SIGNAL( pressed() ), SLOT( menuButtonPressed() ) );
                layout->addWidget( button[MenuButton] );
              }
            break;

            // OnAllDesktops button
          case 'S' :
            if ( !button[OnAllDesktopsButton] )
              {
                button[OnAllDesktopsButton] = new nVIDIAButton( this, "on_all_desktops",
                                              OnAllDesktopsButton, isOnAllDesktops()?i18n("Not On All Desktops"):i18n("On All Desktops"), Settings_Param.btnposflag );
                if(isOnAllDesktops())
                  button[OnAllDesktopsButton]->toggle();
                connect( button[OnAllDesktopsButton], SIGNAL( clicked() ), SLOT( toggleOnAllDesktops() ) );
                layout->addWidget( button[OnAllDesktopsButton] );
              }
            break;

            // Help button
          case 'H' :
            if ( !button[HelpButton] && providesContextHelp() )
              {
                button[HelpButton] = new nVIDIAButton( this, "help", HelpButton, i18n("Help"), Settings_Param.btnposflag );
                connect( button[HelpButton], SIGNAL( clicked() ), SLOT( showContextHelp() ) );
                layout->addWidget( button[HelpButton] );
              }
            break;

            // Minimize button
          case 'I' :
            if ( !button[MinButton] && isMinimizable() )
              {
                button[MinButton] = new nVIDIAButton( this, "minimize", MinButton, i18n("Minimize") );
                connect( button[MinButton], SIGNAL( clicked() ), SLOT( minimize() ) );
                layout->addWidget( button[MinButton] );
              }
            break;

            // Maximize button
          case 'A' :
            if ( !button[MaxButton] && isMaximizable() )
              {
                button[MaxButton] = new nVIDIAButton( this, "maximize", MaxButton, i18n("Maximize"), LeftButton|MidButton|RightButton );
                connect( button[MaxButton], SIGNAL( clicked() ), SLOT( slotMaximize() ) );
                layout->addWidget( button[MaxButton] );
              }
            break;

            // Close button
          case 'X' :
            if ( !button[CloseButton] && isCloseable() )
              {
                button[CloseButton] = new nVIDIAButton( this, "close", CloseButton, i18n("Close") );
                connect( button[CloseButton], SIGNAL( clicked() ), SLOT( closeWindow() ) );
                layout->addWidget( button[CloseButton] );
              }
            break;

            // Additional spacing
          case '_' :
            layout->addSpacing( buttonSpacing );
            break;
          }
      }
#endif
}  

  inline void nVIDIAClient::updateMask()
  {
    if ( !nvidia_initialized )
      return;

    QRegion mask( 0, 0, width(), height() );
    register int r( width() );
    register int b( height()-1 );

    mask -= QRegion( 0, 0, 4, 1 );
    mask -= QRegion( 0, 1, 2, 1 );
    mask -= QRegion( 0, 2, 1, 1 );
    mask -= QRegion( 0, 3, 1, 1 );

    mask -= QRegion( r - 12, 0, 12, 1 );
    mask -= QRegion( r - 9, 1, 9, 1 );
    mask -= QRegion( r - 7, 2, 7, 1 );
    mask -= QRegion( r - 6, 3, 6, 1 );
    mask -= QRegion( r - 5, 4, 5, 1 );
    mask -= QRegion( r - 4, 5, 4, 1 );
    mask -= QRegion( r - 3, 6, 3, 2 );
    mask -= QRegion( r - 2, 8, 2, 3 );
    mask -= QRegion( r - 1, 10, 1, 14 );

    mask -= QRegion(0, b-0, 2, b-1);
    mask -= QRegion(0, b-1, 1, b-1);

    mask -= QRegion(r - 2, b-0, 2, b-1);
    mask -= QRegion(r - 1, b-1, 1, b-1);
  
    setMask( mask );
    maskDirty = false;
  }

  void nVIDIAClient::captionChange()
  {
    maskDirty = true;
    bool active = isActive();
    captionBufferDirty = true;
    widget()->update(clientHandler->tile( TitleLeft, active )->width() + iconAppSpacing, ((clientHandler->tile(TitleCenter, active )->height() - 16)/2)+1, 16, 16);
#ifdef RIGHTRISTATE

    switch (ButtonsLeft)
      {
      case 3:
        widget()->update(clientHandler->tile( TitleLeft, active )->width() + iconAppSpacing + 16 + textSpacing, 0, width()-(clientHandler->tile( TitleRight1, active )->width()+clientHandler->tile( TitleLeft, active )->width()+ iconAppSpacing +16 + textSpacing), clientHandler->tile(TitleCenter, active )->height());
        break;
      case 4:
        widget()->update(clientHandler->tile( TitleLeft, active )->width() + iconAppSpacing + 16 + textSpacing, 0, width()-(clientHandler->tile( TitleRight2, active )->width()+clientHandler->tile( TitleLeft, active )->width()+ iconAppSpacing +16 + textSpacing), clientHandler->tile(TitleCenter, active )->height());
        break;
      case 5:
        widget()->update(clientHandler->tile( TitleLeft, active )->width() + iconAppSpacing + 16 + textSpacing, 0, width()-(clientHandler->tile( TitleRight3, active )->width()+clientHandler->tile( TitleLeft, active )->width()+ iconAppSpacing +16 + textSpacing), clientHandler->tile(TitleCenter, active )->height());
        break;
      }
#else

    widget()->update(clientHandler->tile( TitleLeft, active )->width() + iconAppSpacing + 16 + textSpacing, 0, width()-(clientHandler->tile( TitleRight, active )->width()+clientHandler->tile( TitleLeft, active )->width()+ iconAppSpacing +16 + textSpacing), clientHandler->tile(TitleCenter, active )->height());
#endif

  }

  void nVIDIAClient::iconChange()
  {
    bool active = isActive();
    if ( clientHandler->showAppIcons() )
      {
        if ( activeIcon )
          delete activeIcon;
        if ( inactiveIcon )
          delete inactiveIcon;
        activeIcon = inactiveIcon = NULL;
        iconBufferDirty = true;
        widget()->update(clientHandler->tile( TitleLeft, active )->width()+iconAppSpacing, ((clientHandler->tile(TitleCenter, active )->height() - 16)/2)+1, 16, 16);
      }
  }

  inline void nVIDIAClient::drawAppIcon(QPainter &p)
  {
    if ( !nvidia_initialized )
      return;
    QPixmap *icon = NULL;
    bool active = isActive();
    if ( clientHandler->showAppIcons() )
      {
        if ( active )
          {
            if ( ! activeIcon )
              activeIcon = new QPixmap( this->icon().pixmap( QIconSet::Small, QIconSet::Normal )); // FRAME
            icon = activeIcon;
          }
        else
          {
            if ( ! inactiveIcon )
              {
                QImage img = this->icon().pixmap( QIconSet::Small, QIconSet::Normal ).convertToImage();
                if (Settings_Param.Transparency)
                  {
                    KIconEffect::semiTransparent(img);
                  }
                if (Settings_Param.effects == "togray")
                  {
                    KIconEffect::toGray(img, Settings_Param.effectvalues/100);
                  }
                else if (Settings_Param.effects == "colorize")
                  {
                    KIconEffect::colorize(img, Settings_Param.IconEffectColor, Settings_Param.effectvalues/100);
                  }
                else if (Settings_Param.effects == "gamma")
                  {
                    KIconEffect:: toGamma(img, Settings_Param.effectvalues/100);
                  }
                else if (Settings_Param.effects == "desaturate")
                  {
                    KIconEffect::deSaturate(img, Settings_Param.effectvalues/100);
                  }
                inactiveIcon = new QPixmap( img );
              }
            icon = inactiveIcon;
          }
        p.drawPixmap( clientHandler->tile( TitleLeft, active )->width()+iconAppSpacing, ((clientHandler->tile(TitleCenter, active )->height() - icon->height())/2)+1, *icon, 0, 0, icon->width(), icon->height());
        iconBufferDirty = false;
      }
  }

  inline void nVIDIAClient::drawCaptionText(QPainter & p)
  {
    if ( !nvidia_initialized )
      return;
    int flags = 0;
    bool active = isActive();
    p.setFont( clientHandler->options()->font( active ) );
    // Draw the titlebar text
    if (Settings_Param.textpos == "left")
      {
        flags = AlignLeft | AlignVCenter | SingleLine;
      }
    else if (Settings_Param.textpos == "center")
      {
        flags = AlignCenter | AlignVCenter | SingleLine;
      }
    else if (Settings_Param.textpos == "right")
      {
        flags = AlignRight | AlignVCenter | SingleLine;
      }

    if ( clientHandler->useShadowedText() )
      {
        p.translate( QApplication::reverseLayout() ? -1 : 1, 1 );
        p.setPen(QColor(Settings_Param.ShadowColor));
#ifdef RIGHTRISTATE

        switch (ButtonsLeft)
          {
          case 3:
            p.drawText( (clientHandler->tile( TitleLeft, active )->width() + iconAppSpacing +16 + textSpacing), 0, width()-(clientHandler->tile( TitleRight1, active )->width()+clientHandler->tile( TitleLeft, active )->width()+ iconAppSpacing +16 + textSpacing), clientHandler->tile( TitleCenter, active )->height(), flags, caption() );
            break;
          case 4:
            p.drawText( (clientHandler->tile( TitleLeft, active )->width() + iconAppSpacing +16 + textSpacing), 0, width()-(clientHandler->tile( TitleRight2, active )->width()+clientHandler->tile( TitleLeft, active )->width()+ iconAppSpacing +16 + textSpacing), clientHandler->tile( TitleCenter, active )->height(), flags, caption() );
            break;
          case 5:
            p.drawText( (clientHandler->tile( TitleLeft, active )->width() + iconAppSpacing +16 + textSpacing), 0, width()-(clientHandler->tile( TitleRight3, active )->width()+clientHandler->tile( TitleLeft, active )->width()+ iconAppSpacing +16 + textSpacing), clientHandler->tile( TitleCenter, active )->height(), flags, caption() );
            break;
          }

#else

        p.drawText( (clientHandler->tile( TitleLeft, active )->width() + iconAppSpacing +16 + textSpacing), 0, width()-(clientHandler->tile( TitleRight, active )->width()+clientHandler->tile( TitleLeft, active )->width()+ iconAppSpacing +16 + textSpacing), clientHandler->tile( TitleCenter, active )->height(), flags, caption() );
#endif

        p.translate( QApplication::reverseLayout() ? 1 : -1, -1 );
      }
    p.setPen( options()->color( ColorFont, active ) );
#ifdef RIGHTRISTATE

    switch (ButtonsLeft)
      {
      case 3:
        p.drawText( (clientHandler->tile( TitleLeft, active )->width() + iconAppSpacing +16 + textSpacing), 0, width()-(clientHandler->tile( TitleRight1, active )->width()+clientHandler->tile( TitleLeft, active )->width()+ iconAppSpacing +16 + textSpacing), clientHandler->tile( TitleCenter, active )->height(), flags, caption() );
        break;
      case 4:
        p.drawText( (clientHandler->tile( TitleLeft, active )->width() + iconAppSpacing +16 + textSpacing), 0, width()-(clientHandler->tile( TitleRight2, active )->width()+clientHandler->tile( TitleLeft, active )->width()+ iconAppSpacing +16 + textSpacing), clientHandler->tile( TitleCenter, active )->height(), flags, caption() );
        break;
      case 5:
        p.drawText( (clientHandler->tile( TitleLeft, active )->width() + iconAppSpacing +16 + textSpacing), 0, width()-(clientHandler->tile( TitleRight3, active )->width()+clientHandler->tile( TitleLeft, active )->width()+ iconAppSpacing +16 + textSpacing), clientHandler->tile( TitleCenter, active )->height(), flags, caption() );
        break;
      }
#else

    p.drawText( (clientHandler->tile( TitleLeft, active )->width() + iconAppSpacing +16 + textSpacing), 0, width()-(clientHandler->tile( TitleRight, active )->width()+clientHandler->tile( TitleLeft, active )->width()+ iconAppSpacing +16 + textSpacing), clientHandler->tile( TitleCenter, active )->height(), flags, caption() );
#endif

    captionBufferDirty = false;
  }

#ifdef RIGHTRISTATE

  inline int nVIDIAClient::ButtonsOnLeft()
  {
    Settings_Param.numbuttons = 2;
    QString s = options()->titleButtonsRight();
    for(register uint x = 0; x < options()->titleButtonsRight().length(); x++ )
      {
        if(s[x].latin1() == 'I' && Settings_Param.mine)
          Settings_Param.numbuttons++;
        else if (s[x].latin1() == 'A' && Settings_Param.maxe)
          Settings_Param.numbuttons++;
        else if (s[x].latin1() == 'X' && Settings_Param.closee)
          Settings_Param.numbuttons++;
      }
    return Settings_Param.numbuttons;
  }
#endif

  void nVIDIAClient::activeChange()
  {
    maskDirty = true;
    captionBufferDirty = true;
    iconBufferDirty = true;
    widget()->update();
#ifdef AVATARENABLED

    btt->update();
#endif

    for (register int i=0; i < NumButtons; i++ )
      if ( button[i] )
        button[i]->update();
  }

  void nVIDIAClient::maximizeChange()
  {
    captionBufferDirty = true;
    iconBufferDirty = true;
    if (Settings_Param.bubbletype != 0)
      {
        if ( clientHandler->largeCaptionBubbles() )
          {
            if ( maximizeMode() & MaximizeVertical )
              {
                // We've been maximized - shrink the titlebar by 3 pixels
                if (Settings_Param.bubbletype !=0 )
                  topSpacer->changeSize( 10, clientHandler->tile(CaptionLargeCenter, isActive())->height()-clientHandler->tile(TitleCenter, isActive())->height(), QSizePolicy::Expanding, QSizePolicy::Minimum );
                else
                  topSpacer->changeSize( 10, 0, QSizePolicy::Expanding, QSizePolicy::Minimum );
                maskDirty = true;
                widget()->layout()->activate();
                widget()->update();
              }
          }
      }
    if ( button[ MaxButton ] )
      {
        QToolTip::remove
          ( button[ MaxButton ] );
        QToolTip::add
          ( button[ MaxButton ], maximizeMode() == MaximizeFull ? i18n("Restore") : i18n("Maximize") );
        button[ MaxButton ]->update();
      }
    widget()->update();
#ifdef AVATARENABLED

    btt->update();
#endif

  }


  void nVIDIAClient::desktopChange()
  {
  #ifndef CUSTOMBUTTONS
    if ( button[ OnAllDesktopsButton ] )
      {
        QToolTip::remove
          ( button[ OnAllDesktopsButton ] );
        QToolTip::add
          ( button[ OnAllDesktopsButton ], isOnAllDesktops() ? i18n("Not On All Desktops") : i18n("On All Desktops") );
      }
#endif      
  }


  void nVIDIAClient::menuButtonPressed()
  {
  #ifndef CUSTOMBUTTONS
    QPoint menuPoint ( button[MenuButton]->rect().bottomLeft().x() - 6,
                       button[MenuButton]->rect().bottomLeft().y() + 3 );
    KDecorationFactory* f = factory();
    showWindowMenu( button[MenuButton]->mapToGlobal( menuPoint ));
    if( !f->exists( this )) // 'this' was destroyed
      return;
    button[MenuButton]->setDown(false);
#endif    
  }

  void nVIDIAClient::slotMaximize()
  {
    switch ( button[ MaxButton ]->lastButton() )
      {
      case MidButton:
        maximize( maximizeMode() ^ MaximizeVertical );
        break;

      case RightButton:
        maximize( maximizeMode() ^ MaximizeHorizontal );
        break;

      case LeftButton:
        maximize( maximizeMode() == MaximizeFull ? MaximizeRestore : MaximizeFull );
        break;
      }
  }

#ifdef AVATARENABLED

  void nVIDIAClient::AvatarAction()
  {
    KProcess* proc =new KProcess();
    if(Avatar_Params.avatarfunctype == "Konqueror")
      {
        *proc << "konqueror";
        *proc << "--profile" << "webbrowsing" << "URL" << Avatar_Params.avatarfuncURL;
      }
    else if(Avatar_Params.avatarfunctype == "Kontact")
      *proc << "kontact";
    else if(Avatar_Params.avatarfunctype == "Kopete")
      *proc << "kopete";
    else if(Avatar_Params.avatarfunctype == "Kaffeine")
      *proc << "kaffeine";
    else if(Avatar_Params.avatarfunctype == "AmaroK")
      *proc << "amarok";
    else if(Avatar_Params.avatarfunctype == "KControl")
      *proc << "kcontrol";
    else if(Avatar_Params.avatarfunctype == "Juk")
      *proc << "juk";
    else
      return;
    proc->start();
  }
#endif

  inline void nVIDIAClient::paintEvent( QPaintEvent *e )
  {
    if ( !nvidia_initialized )
      return;

    QPainter p( widget());
    QRect updateRect( e->rect() );
    bool active = isActive();

    int titleBaseY         =  0;
    int titleBarHeight     = clientHandler->titleBarHeight( false );
    int grabBarHeight      = clientHandler->grabBarHeight();
    int leftBorderWidth    = clientHandler->tile( BorderLeft, active )->width();
    int rightBorderWidth   = clientHandler->tile( BorderRight, active )->width();

    if ( maskDirty )
      updateMask();

    // Titlebar
    // -----------------------------------------------------------------------
    if ( updateRect.y() < titleBarHeight )
      {
        // Top left corner
        if ( updateRect.x() < clientHandler->tile( TitleLeft, active )->width() )
          p.drawPixmap( 0, titleBaseY, *clientHandler->tile( TitleLeft, active ) );
      }
    // Caption bubble
    if (Settings_Param.bubbletype != 0)
      {
        int tw = p.fontMetrics().width( caption()) +30+16 + iconSpacing;//caption width for bubble(caption width + space for text + icon)
        //left caption border
        p.drawPixmap(clientHandler->tile( TitleLeft, active )->width()+15, titleBaseY,*clientHandler->tile( (Settings_Param.smallCaptionBubbles ? CaptionSmallLeft : CaptionLargeLeft), active ));
        //center bubble
        p.drawTiledPixmap(clientHandler->tile( TitleLeft, active )->width()+15+clientHandler->tile((Settings_Param.smallCaptionBubbles ? CaptionSmallLeft : CaptionLargeLeft), active )->width(), titleBaseY, tw, clientHandler->tile((Settings_Param.smallCaptionBubbles ? CaptionSmallCenter : CaptionLargeCenter), active )->height(), *clientHandler->tile((Settings_Param.smallCaptionBubbles ? CaptionSmallCenter : CaptionLargeCenter), active ));
        //right bubble
        p.drawPixmap(clientHandler->tile( TitleLeft, active )->width()+15+tw, titleBaseY, *clientHandler->tile( (Settings_Param.smallCaptionBubbles ? CaptionSmallRight : CaptionLargeRight), active ));
      }
    else
      //center pixmap
#ifdef RIGHTRISTATE

      switch (ButtonsLeft)
        {
        case 3:
          p.drawTiledPixmap(clientHandler->tile( TitleLeft, active )->width(), titleBaseY, width()-(clientHandler->tile( TitleRight1, active )->width()+clientHandler->tile( TitleLeft, active )->width()), clientHandler->tile( TitleCenter, active )->height(),  *clientHandler->tile( TitleCenter, active ));
          break;
        case 4:
          p.drawTiledPixmap(clientHandler->tile( TitleLeft, active )->width(), titleBaseY, width()-(clientHandler->tile( TitleRight2, active )->width()+clientHandler->tile( TitleLeft, active )->width()), clientHandler->tile( TitleCenter, active )->height(),  *clientHandler->tile( TitleCenter, active ));
          break;
        case 5:
          p.drawTiledPixmap(clientHandler->tile( TitleLeft, active )->width(), titleBaseY, width()-(clientHandler->tile( TitleRight3, active )->width()+clientHandler->tile( TitleLeft, active )->width()), clientHandler->tile( TitleCenter, active )->height(),  *clientHandler->tile( TitleCenter, active ));
          break;
        }

#else

      p.drawTiledPixmap(clientHandler->tile( TitleLeft, active )->width(), titleBaseY, width()-(clientHandler->tile( TitleRight, active )->width()+clientHandler->tile( TitleLeft, active )->width()), clientHandler->tile( TitleCenter, active )->height(),  *clientHandler->tile( TitleCenter, active ));
#endif
    // Top right corner
#ifdef RIGHTRISTATE

    switch (ButtonsLeft)
      {
      case 3:
        if ( updateRect.right() >= width() - clientHandler->tile( TitleRight1, active )->width() )
          p.drawPixmap( width() - clientHandler->tile( TitleRight1, active )->width(), titleBaseY, *clientHandler->tile( TitleRight1, active ) );
        break;
      case 4:
        if ( updateRect.right() >= width() - clientHandler->tile( TitleRight2, active )->width() )
          p.drawPixmap( width() - clientHandler->tile( TitleRight2, active )->width(), titleBaseY, *clientHandler->tile( TitleRight2, active ) );
        break;
      case 5:
        if ( updateRect.right() >= width() - clientHandler->tile( TitleRight3, active )->width() )
          p.drawPixmap( width() - clientHandler->tile( TitleRight3, active )->width(), titleBaseY, *clientHandler->tile( TitleRight3, active ) );
        break;
      }

#else

    if ( updateRect.right() >= width() - clientHandler->tile( TitleRight, active )->width() )
      p.drawPixmap( width() - clientHandler->tile( TitleRight, active )->width(), titleBaseY, *clientHandler->tile( TitleRight, active ) );
#endif

    drawAppIcon(p);
    drawCaptionText(p);
    // Borders
    // -----------------------------------------------------------------------
    if ( updateRect.bottom() >= titleBarHeight && updateRect.top() < height() - grabBarHeight )
      {
        register int top    = QMAX( titleBarHeight, updateRect.top() );
        register int bottom = QMIN( updateRect.bottom(), height() - grabBarHeight );

        // Left border
        if ( updateRect.x() < leftBorderWidth )
          p.drawTiledPixmap( 0, top, leftBorderWidth, bottom - top + 1,
                             *clientHandler->tile( BorderLeft, active ) );

        // Right border
        if ( e->rect().right() > width() - rightBorderWidth - 1 )
          p.drawTiledPixmap( width() - rightBorderWidth, top, rightBorderWidth,
                             bottom - top + 1, *clientHandler->tile( BorderRight, active ) );
      }

    // Bottom grab bar
    // -----------------------------------------------------------------------
    if ( updateRect.bottom() >= height() - grabBarHeight )
      {
        // Bottom left corner
        if ( updateRect.x() < 9 )
          p.drawPixmap( 0, height() - grabBarHeight,
                        *clientHandler->tile( GrabBarLeft, active ) );

        // Space between the left corner and the right corner
        if ( updateRect.x() < width() - 9 )
          {
            register int x1 = QMAX( 9, updateRect.x() );
            register int x2 = QMIN( width() - 9, updateRect.right() );

            p.drawTiledPixmap( x1, height() - grabBarHeight, x2 - x1 + 1,
                               grabBarHeight, *clientHandler->tile( GrabBarCenter, active ) );
          }

        // Bottom right corner
        if ( updateRect.right() > width() - 9 )
          p.drawPixmap( width() - 9, height() - grabBarHeight,
                        *clientHandler->tile( GrabBarRight, active ) );
      }
  }

  void nVIDIAClient::resizeEvent( QResizeEvent *e )
  {
    // FRAME	Client::resizeEvent( e );
    //QRect r( captionRect );
    //if ( r.size() != captionRect.size() )
    maskDirty = true;
    captionBufferDirty = true;
    iconBufferDirty = true;
    if ( widget()->isVisible() )
      {
        widget()->update( widget()->rect() );
        register int dx = 0;
        register int dy = 0;

        if ( e->oldSize().width() != width() )
          dx = 32 + QABS( e->oldSize().width() -  width() );

        if ( e->oldSize().height() != height() )
          dy = 8 + QABS( e->oldSize().height() -  height() );

        if ( dy )
          widget()->update( 0, height() - dy + 1, width(), dy );

        if ( dx )
          {
            widget()->update( width() - dx + 1, 0, dx, height() );
            widget()->update( QRect( QPoint(4,4), titlebar->geometry().bottomLeft() - QPoint(1,0) ) );
            widget()->update( QRect( titlebar->geometry().topRight(), QPoint( width() - 4,
                                     titlebar->geometry().bottom() ) ) );
#ifdef AVATARENABLED

            btt->update();
#endif
            // Titlebar needs no paint event
            QApplication::postEvent( this, new QPaintEvent( titlebar->geometry(), FALSE ) );
          }
      }
  }


  void nVIDIAClient::mouseDoubleClickEvent( QMouseEvent *e )
  {
    if ( QRect( 0, 0, width(), clientHandler->titleBarHeight( false ) ).contains( e->pos() ) )
      titlebarDblClickOperation();
  }


  nVIDIAClient::Position nVIDIAClient::mousePosition( const QPoint &p ) const
    {
      register int titleBaseY = 0;//(largeTitlebar ? 3 : 0);
      register int leftBorder   = clientHandler->tile( BorderLeft, true )->width();
      register int rightBorder  = width() - clientHandler->tile( BorderRight, true )->width() - 1;
      register int bottomBorder = height() - clientHandler->grabBarHeight() - 1;
      register int bottomCornerSize = 3*clientHandler->tile( BorderRight, true )->width()/2 + 24;

      // Test if the mouse is over the titlebar area
      if ( p.y() < titleBaseY + 11 )
        {
          // Test for the top left corner
          if ( p.x() < leftBorder + 11 )
            {
              if ( (p.y() < titleBaseY + 3 && p.x() < leftBorder + 11) ||
                   (p.y() < titleBaseY + 6 && p.x() < leftBorder + 6) ||
                   (p.y() < titleBaseY + 11 && p.x() < leftBorder + 3) )
                return PositionTopLeft;
            }

          // Test for the top right corner
          if ( p.x() > rightBorder - 11 )
            {
              if ( (p.y() < titleBaseY + 3 && p.x() > rightBorder - 11) ||
                   (p.y() < titleBaseY + 6 && p.x() > rightBorder - 6) ||
                   (p.y() < titleBaseY + 11 && p.x() > rightBorder - 3) )
                return PositionTopRight;
            }

          // Test for the top border
          if ( p.y() <= 3 || (p.y() <= titleBaseY+3 ))
            return PositionTop;

          // The cursor must be over the center of the titlebar.
          return PositionCenter;
        }

      // Test the sides
      else if ( p.y() < bottomBorder )
        {
          // Test for the left side
          if ( p.x() < leftBorder )
            {
              if ( p.y() < height() - bottomCornerSize )
                return PositionLeft;
              else
                return PositionBottomLeft;
            }

          // Test for the right side
          else if ( p.x() > rightBorder )
            {
              if ( p.y() < height() - bottomCornerSize )
                return PositionRight;
              else
                return PositionBottomRight;
            }

          // The cursor must be over the center of the window
          return PositionCenter;
        }

      // Test the grab bar / bottom border
      else
        {
          // Test for the bottom left corner
          if ( p.x() < bottomCornerSize )
            return PositionBottomLeft;

          // Test for the bottom right corner
          else if ( p.x() > width() - bottomCornerSize - 1 )
            return PositionBottomRight;

          // The cursor must be over the bottom border
          return PositionBottom;
        }

      // We should never get here
      return PositionCenter;
    }


  void nVIDIAClient::resize( const QSize& s )
  {
    widget()->resize( s );
  }


  void nVIDIAClient::borders( register int& left, register int& right, register int& top, register int& bottom ) const
    {
      register int titleBarHeight     = clientHandler->titleBarHeight( clientHandler->largeCaptionBubbles() );
      register int grabBarHeight      = clientHandler->grabBarHeight();
      register int leftBorderWidth    = clientHandler->tile( BorderLeft, isActive() )->width();
      register int rightBorderWidth   = clientHandler->tile( BorderRight, isActive() )->width();

      left   = leftBorderWidth;
      right  = rightBorderWidth;
      top    = titleBarHeight;
      bottom = grabBarHeight;

      if ( ( maximizeMode() & MaximizeHorizontal ) && !options()->moveResizeMaximizedWindows())
        left = right = 0;
      if( maximizeMode() & MaximizeVertical)
        {
          top = clientHandler->titleBarHeight( false );
          if( !options()->moveResizeMaximizedWindows())
            bottom = 0;
        }
    }


  QSize nVIDIAClient::minimumSize() const
    {
      return widget()->minimumSize();
    }


  bool nVIDIAClient::eventFilter( QObject* o, QEvent* e )
  {
    if ( o != widget() )
      return false;

    switch ( e->type() )
      {
      case QEvent::Resize:
        resizeEvent( static_cast< QResizeEvent* >( e ) );
        return true;

      case QEvent::Paint:
        paintEvent( static_cast< QPaintEvent* >( e ) );
        return true;

      case QEvent::MouseButtonDblClick:
        mouseDoubleClickEvent( static_cast< QMouseEvent* >( e ) );
        return true;

      case QEvent::MouseButtonPress:
        processMousePressEvent( static_cast< QMouseEvent* >( e ) );
        return true;

      default:
        return false;
      }
  }

} // namespace nVIDIA
// -------------------------------------------------------------------------------------------

extern "C"
  {
    KDecorationFactory *create_factory()
    {
      nVIDIA::clientHandler = new nVIDIA::nVIDIAHandler();
      return nVIDIA::clientHandler;
    }
  }
// vim: set noet ts=4 sw=4: