/*
 * $Id: nvidia.h,v 1.12 2003/12/15 15:10:16 lunakl Exp $
 *
 * nVIDIA KWin client (version 0.8)
 *
 * Copyright (C) 2002 Fredrik H�lund <fredrik@kde.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the license, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#ifndef __NVIDIA_H
#define __NVIDIA_H

#define RIGHTRISTATE
#define AVATARENABLED
#define CUSTOMBUTTONS

#include <qbutton.h>
#include <kdecoration.h>
#include <kdecorationfactory.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qlayout.h>
#include "tiles.h"
#include <qstring.h>
#include <qcolor.h>
#include <qbitmap.h>

class QSpacerItem;

namespace nVIDIA
  {
#ifdef RIGHTRISTATE
  enum TilePixmap  { TitleLeft=0, TitleCenter, TitleRight, TitleRight1, TitleRight2, TitleRight3, CaptionSmallLeft, CaptionSmallCenter, CaptionSmallRight, CaptionLargeLeft, CaptionLargeCenter, CaptionLargeRight, GrabBarLeft, GrabBarCenter, GrabBarRight, BorderLeft, BorderRight, NumTiles };
#else

  enum TilePixmap  { TitleLeft=0, TitleCenter, TitleRight, CaptionSmallLeft, CaptionSmallCenter, CaptionSmallRight, CaptionLargeLeft, CaptionLargeCenter, CaptionLargeRight, GrabBarLeft, GrabBarCenter, GrabBarRight, BorderLeft, BorderRight, NumTiles };
#endif

#ifdef CUSTOMBUTTONS

  enum Button      { HelpButton=0, MinButton, MaxButton, CloseButton, NumButtons };
  enum ButtonDeco  { Help=0, Minimize, Maximize, Restore, Close, NumButtonDecos };
#else

  enum Button      { MenuButton=0, OnAllDesktopsButton, HelpButton, MinButton, MaxButton, CloseButton, NumButtons };
  enum ButtonDeco  { Menu=0, OnAllDesktops, NotOnAllDesktops, Help, Minimize, Maximize, Restore, Close, NumButtonDecos };
#endif

  enum TileButtons {ButtonRound=0, ButtonSquare, NumButton};

  struct 
    {
      bool showIcons, shadowedText, largeGrabBars, smallCaptionBubbles, Transparency, closee, maxe, mine;
      int shadowtype, effectvalues, bubbletype, topSpacing, btnposflag, numbuttons;
      QString textpos, coloroption, effects;
      QColor ShadowColor, IconEffectColor;
    }Settings_Param;
#ifdef AVATARENABLED

  struct 
    {
      QString avatarfuncURL, avatarpix, avatarfunctype;
      int avatarfunc, avatar;
    }Avatar_Params;
#endif

  class nVIDIAHandler : public KDecorationFactory
    {
    public:
      nVIDIAHandler();
      ~nVIDIAHandler();

      virtual QValueList< BorderSize > borderSizes() const;
      virtual bool reset( unsigned long changed );
      virtual KDecoration* createDecoration( KDecorationBridge* );

      bool showAppIcons() const
        {
          return Settings_Param.showIcons;
        }
      bool useShadowedText() const
        {
          return Settings_Param.shadowedText;
        }
      bool largeCaptionBubbles() const
        {
          return !Settings_Param.smallCaptionBubbles;
        }

      int titleBarHeight( bool large ) const
        {
          if (Settings_Param.bubbletype != 0)
            {
              return ( large ? activeTiles[CaptionLargeCenter]->height()
                       : activeTiles[CaptionSmallCenter]->height() );
            }
          else
            {
              return activeTiles[TitleCenter]->height();
            }
        }

      int grabBarHeight() const
        {
          return activeTiles[GrabBarCenter]->height();
        }

      inline const QPixmap *tile( TilePixmap tilePix, bool active ) const;
      inline const QPixmap *btile(TileButtons buttonPix, bool active) const;
      inline const QPixmap *decotile(ButtonDeco tiledeco, bool active) const;
    protected:
      void readConfig();
      void createPixmaps();
      void destroyPixmaps();

      void addWidth  (int width,  QPixmap *&pix, bool left, QPixmap *bottomPix);
      void addHeight (int height, QPixmap *&pix);
      void flip( QPixmap *&, QPixmap *& );
      void flip( QPixmap *& );
      void pretile( QPixmap *&, int, Qt::Orientation );
      QImage  *loadImage( const QString &);//, const QColor & );
      QPixmap *loadPixmap( const QString &);//, const QColor & );
      nVIDIAImageDb *imageDb;

      QPixmap *activeTiles[ NumTiles ];
      QPixmap *inactiveTiles[ NumTiles ];
      QPixmap *activebuttonDecos[ NumButtonDecos ];
      QPixmap *inactivebuttonDecos[NumButtonDecos];
      QPixmap *activeButtons[NumButton];
      QPixmap *inactiveButtons[NumButton];
    }
  ; // class nVIDIAHandler

  class nVIDIAClient;
  class nVIDIAButton : public QButton
    {
    public:
#ifdef CUSTOMBUTTONS

      nVIDIAButton( nVIDIAClient *, const char *, Button, const QString & );
#endif

      nVIDIAButton( nVIDIAClient *, const char *, Button, const QString &, const int realizeBtns = LeftButton );
      ~nVIDIAButton();

      int lastButton() const
        {
          return lastbutton;
        }

    private:
      void enterEvent( QEvent * );
      void leaveEvent( QEvent * );
      void mousePressEvent( QMouseEvent * );
      void mouseReleaseEvent( QMouseEvent * );
      void drawButton( QPainter * );
      void updateMask();
    private:
      nVIDIAClient *client;
      Button button;
      bool hover;
      int lastbutton;
      int realizeButtons;
    }
  ; // class nVIDIAButton

#ifdef AVATARENABLED

  class nVIDIAAvatarButton : public QButton
    {
    public:
      nVIDIAAvatarButton(nVIDIAClient *, const char *, const QString &);
      ~nVIDIAAvatarButton();
      QImage LoadAvatar();
    private:
      void enterEvent( QEvent * );
      void leaveEvent( QEvent * );
      void drawButton( QPainter * );
      void updateMask();
      QImage FastScaleImage(const QString &, int, int);
    private:
      nVIDIAClient *client;
      bool over;
    };
#endif

  class nVIDIAClient : public KDecoration
    {
      Q_OBJECT

    public:

      nVIDIAClient( KDecorationBridge* bridge, KDecorationFactory* factory );
      ~nVIDIAClient();
#ifdef RIGHTRISTATE

      int ButtonsOnLeft();
#endif

      virtual void init();
      virtual void reset( unsigned long changed );
      virtual Position mousePosition( const QPoint& p ) const;
      virtual void borders( int& left, int& right, int& top, int& bottom ) const;
      virtual void resize( const QSize& s );
      virtual QSize minimumSize() const;
      virtual bool eventFilter( QObject* o, QEvent* e );
      virtual void activeChange();
      virtual void captionChange();
      virtual void maximizeChange();
      virtual void desktopChange();
      virtual void shadeChange()
      {}
      ;

    private:
      void createLayout();
      void addButtons( QBoxLayout*, const QString & );
#ifdef AVATARENABLED

      void addAvatar();
#endif

      void updateMask(); // FRAME
      void iconChange();
      void drawAppIcon(QPainter &);
      void drawCaptionText(QPainter &);
      void resizeEvent( QResizeEvent *); // FRAME
      void paintEvent( QPaintEvent *); // FRAME
      void mouseDoubleClickEvent( QMouseEvent * ); // FRAME
      inline int width() const
        {
          return widget()->width();
        }

      inline int height() const
        {
          return widget()->height();
        }

      inline bool maximizedVertical() const
        {
          return ( maximizeMode() & MaximizeVertical );
        }

    private slots:
      void menuButtonPressed();
      void slotMaximize();
#ifdef AVATARENABLED

      void AvatarAction();
#endif

    private:
      QSpacerItem   *topSpacer, *titlebar;
      nVIDIAButton *button[ NumButtons ];
      QPixmap       *activeIcon, *inactiveIcon;
    bool maskDirty:
      1;
    bool captionBufferDirty:
      1;
    bool iconBufferDirty:
      1;
    bool largeCaption:
      1;
#ifdef RIGHTRISTATE

      int ButtonsLeft;
#endif
#ifdef AVATARENABLED

      nVIDIAAvatarButton* btt ;
#endif
#ifdef CUSTOMBUTTONS

      nVIDIAButton* bttHelp ;
#endif

    }
  ; // class nVIDIAClient

} // namespace nVIDIA

#endif // ___NVIDIA_H

// vim: set noet ts=4 sw=4:
